# config.py
#
# Copyright (C) 2014 Tim Marston <tim@edm.am>
#
# This file is part of stdhome (hereafter referred to as "this program").
# See http://ed.am/dev/stdhome for more information.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import os, re, errno
from file_matcher import FileMatcher
import the


class Config:


	def __init__( self ):
		self.symlinks = None
		self.ignores = None
		self.default_vcs = 'bzr'
		self.read_config( the.config_file )

	def read_config( self, config_file ):

		# read file
		if the.verbose >= 1: print "reading config: %s" % ( config_file )
		file = os.path.expanduser( config_file )
		try:
			with open( file ) as f:
				lines = f.readlines()
		except IOError as e:
			if e.errno != errno.ENOENT: raise
			lines = list()

		# parse config
		config = dict();
		for line in lines:

			# clean up line and skip empty lines and comments
			line = line.strip();
			if not len( line ): continue
			if line[ :1 ] == '#': continue

			# if section
			matches = re.search( '^\[(.*)\]$', line );
			if matches:
				section = matches.group( 1 )
				continue
			matches = re.search( '^([a-z0-9_-]+)\\s*=\\s*(.*)$', line, re.I )
			if matches:
				name = matches.group( 1 )
				value = matches.group( 2 )

				# parse value
				matches = re.search(
					'^(["\'])((?:\\.|[^\\1])*)\\1(?:\\s*#.*)?$', value )
				if matches:
					value = matches.group( 2 )
				else:
					matches = re.search( '^([^#]+?)(?:\\s*#.*)?$', value )
					if( matches ):
						value = matches.group( 1 )
				config[ "%s.%s" % ( section, name ) ] = value
				#print "  %s.%s = %s" % ( section, name, value )

		# use config
		if 'stdhome.home-dir' in config:
			the.home_dir = config[ 'stdhome.home-dir' ]
		if 'stdhome.default-vcs' in config:
			self.default_vcs = config[ 'stdhome.default-vcs' ]

		# read file matcher lists
		self.symlinks = FileMatcher( config_file, 'symlink' )
		self.ignores = FileMatcher( config_file, 'ignore' )


	def to_bool( self, value ):
		value = value.lower()
		return value == 't' or value == 'true' or value == 'yes' or \
			( re.search( '^[0-9]+$', value ) and int( value ) != 0 )
