# repo.py
#
# Copyright (C) 2013 to 2014 Tim Marston <tim@edm.am>
#
# This file is part of stdhome (hereafter referred to as "this program").
# See http://ed.am/dev/stdhome for more information.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import os
import the, util
from vcs.vcs import Vcs


class Repo:


	def __init__( self, name ):
		self.name = name
		self.dir = os.path.join( the.dir, self.name )
		self.full_dir = util.canonicalise_path( self.dir )
		self.vcs = False


	def check_dir_exists( self, should_exist = True ):
		if os.path.exists( self.full_dir ) != should_exist:
			error = 'does not exist' if should_exist else 'already exists'
			raise the.program.FatalError(
				'repository %s: %s' % ( error, self.dir ) )


	def detect_vcs( self ):
		if os.path.exists( self.full_dir ):
			for vcs in Vcs.vcses:
				instance = Vcs.instantiate_vcs( vcs, self.full_dir )
				if instance.has_authority():
					if the.verbose >= 2: print "repo vcs: %s" % vcs
					self.vcs = instance
					break
			if self.vcs == False:
				raise the.program.FatalError(
					'unknown vcs in repository: %s' % self.dir )


	def set_vcs( self, vcs ):
		if Vcs.vcses.count( vcs ) == 1:
			self.vcs = Vcs.instantiate_vcs( vcs, self.full_dir )
			if the.verbose >= 2: print "repo vcs: %s" % vcs
		else:
			raise the.program.FatalError( 'unknown vcs requested: %s' % vcs )


	def __str__( self ):
		return "Repo( %s: %s )" % ( self.name, self.dir )
